import router from './router'
import store from './store'
import { Message } from 'element-ui'
import NProgress from 'nprogress' // progress bar
import 'nprogress/nprogress.css' // progress bar style
import { getToken, setToken } from '@/utils/auth'
import getPageTitle from '@/utils/get-page-title'
NProgress.configure({ showSpinner: false }) // NProgress Configuration
let url = decodeURI(window.location.search || window.location.hash)
//获取哈希地址信息
function queryURLParams(url) {
  let result = {},
    reg1 = /([^?=&#]+)=([^?=&#]+)/g,
    reg2 = /#([^?=&#]+)/g
  url.replace(reg1, (n, x, y) => (result[x] = y))
  url.replace(reg2, (n, x) => (result['HASH'] = x))
  return result
}

let paramsObj = queryURLParams(url)
const whiteList = [] // no redirect whitelist '/login', '/auth-redirect', '/jump'

router.beforeEach(async (to, from, next) => {
  // start progress bar
  NProgress.start()
  // const hasToken = getToken()
  // set page title
  document.title = getPageTitle(to.meta.title, to.meta.zhTitle)
  if (getToken()) {
    if (store.getters.isLock && to.path !== '/lockScreen' && to.path !== '/login') {
      next({
        path: '/lockScreen'
      })
      NProgress.done()
    } else if (to.path === '/login') {
      // if is logged in, redirect to the home page
      next({
        path: '/'
      })
      NProgress.done()
    } else {
      if (to.path === '/home') {
        if (to.query.token) {
          if (process.env.NODE_ENV === 'development') {
            store.commit('user/SET_TOKEN', to.query.token)
            setToken(to.query.token)
          } else {
            let token = JSON.parse(window.localStorage.getItem(paramsObj.token))
            if (token) {
              store.commit('user/SET_TOKEN', token.content)
              setToken(token.content)
            }
          }
        }
      }
      const hasMenu = store.getters.menuList && store.getters.menuList.length > 0
      if (hasMenu) {
        next()
      } else {
        try {
          // get user info
          let res = await store.dispatch('user/getInfo')
          const accessRoutes = await store.dispatch('permission/generateRoutes', res)
          // dynamically add accessible routes
          router.addRoutes(accessRoutes)
          // hack method to ensure that addRoutes is complete
          // set the replace: true, so the navigation will not leave a history record
          next({
            ...to,
            replace: true
          })
        } catch (error) {
          // remove token and go to login page to re-login
          await store.dispatch('user/resetToken')
          if (error && typeof error == 'string') Message.error(error || 'Has Error')
          next(`/login?redirect=${to.path}`)
          NProgress.done()
        }
      }
    }
  } else {
    /* has no token*/
    if (whiteList.indexOf(to.path) !== -1) {
      // in the free login whitelist, go directly
      next()
    } else if (to.path === '/home') {
      if (to.query.token) {
        if (process.env.NODE_ENV === 'development') {
          store.commit('user/SET_TOKEN', to.query.token)
          setToken(to.query.token)
        } else {
          let token = JSON.parse(window.localStorage.getItem(paramsObj.token))
          if (token) {
            store.commit('user/SET_TOKEN', token.content)
            setToken(token.content)
          }
        }
        const hasMenu = store.getters.menuList && store.getters.menuList.length > 0
        if (hasMenu) {
          next()
        } else {
          try {
            let res = await store.dispatch('user/getInfo')
            const accessRoutes = await store.dispatch('permission/generateRoutes')
            console.log(accessRoutes)
            router.addRoutes(accessRoutes)
            next('/home')
          } catch (error) {
            next(`/login`)
            NProgress.done()
          }
        }
      } else {
        next(`/login`)
        NProgress.done()
      }
    } else {
      // other pages that do not have permissionEle to access are redirected to the login page.
      const hasMenu = store.getters.menuList && store.getters.menuList.length > 0
      if (hasMenu) {
        next()
      } else {
        try {
          // get user info
          let res = await store.dispatch('user/getInfo')
          const accessRoutes = await store.dispatch('permission/generateRoutes', res)
          router.addRoutes(accessRoutes)
          next({
            ...to,
            replace: true
          })
        } catch (error) {
          // remove token and go to login page to re-login
          await store.dispatch('user/resetToken')
          if (error && typeof error == 'string') Message.error(error || 'Has Error')
          next(`/login?redirect=${to.path}`)
          NProgress.done()
        }
      }
    }
  }
})

router.afterEach(() => {
  // finish progress bar
  NProgress.done()
})
